package com.perforce.hws.swagger.asciidoc.models;

import io.swagger.models.properties.ArrayProperty;
import io.swagger.models.properties.BaseIntegerProperty;
import io.swagger.models.properties.DateTimeProperty;
import io.swagger.models.properties.MapProperty;
import io.swagger.models.properties.ObjectProperty;
import io.swagger.models.properties.Property;
import io.swagger.models.properties.RefProperty;
import io.swagger.models.properties.StringProperty;

/**
 * Created by tristan on 4/13/16.
 */
public class PropertyHelper implements LinkHelpers {

    private String id;
    private Property property;

    public PropertyHelper(String id, Property property) {
        this.id = id;
        this.property = property;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public Property getProperty() {
        return property;
    }

    public void setProperty(Property property) {
        this.property = property;
    }

    public String getJavaType() {
        if (this.property instanceof StringProperty) {
            return "String";
        } else if (this.property instanceof DateTimeProperty) {
            return "java.util.Date";
        } else if (this.property instanceof ArrayProperty) {
            ArrayProperty arrayProperty = (ArrayProperty)this.property;
            String type = arrayProperty.getItems().getType();
            String jType = "UNKNOWN";
            if (type.equals("string")) {
                jType = "String";
            } else if (type.equals("integer")) {
                jType = "Integer";
            } else if (arrayProperty.getItems() instanceof RefProperty){
                RefProperty refProperty = (RefProperty)arrayProperty.getItems();
                jType = refProperty.getSimpleRef();
            } else if (arrayProperty.getItems() instanceof MapProperty) {
                MapProperty mapProperty = (MapProperty)arrayProperty.getItems();
                jType = mapProperty.getType();
            }
            return "List<" + jType + ">";
        } else if (this.property instanceof BaseIntegerProperty) {
            return "Integer";
        } else if (this.property instanceof RefProperty) {
            RefProperty refProperty = (RefProperty)this.property;
            return refProperty.getSimpleRef();
        } else if (this.property instanceof ObjectProperty) {
            // This is likely a placeholder around Swagger funkiness
            return "Object";
        }
        return "UNKNOWN";
    }

    public String getJavaTypeRef() {
        if (this.property instanceof StringProperty) {
            return "String";
        } else if (this.property instanceof DateTimeProperty) {
            return "java.util.Date";
        } else if (this.property instanceof ArrayProperty) {
            ArrayProperty arrayProperty = (ArrayProperty)this.property;
            String type = arrayProperty.getItems().getType();
            String jType = "UNKNOWN";
            if (type.equals("string")) {
                jType = "String";
            } else if (type.equals("integer")) {
                jType = "Integer";
            } else if (arrayProperty.getItems() instanceof RefProperty){
                RefProperty refProperty = (RefProperty)arrayProperty.getItems();
                jType = javaModelRef(refProperty.getSimpleRef());
            } else if (arrayProperty.getItems() instanceof MapProperty) {
                MapProperty mapProperty = (MapProperty)arrayProperty.getItems();
                jType = mapProperty.getType();
            }
            return "List<" + jType + ">";
        } else if (this.property instanceof BaseIntegerProperty) {
            return "Integer";
        } else if (this.property instanceof RefProperty) {
            RefProperty refProperty = (RefProperty)this.property;
            return javaModelRef(refProperty.getSimpleRef());
        } else if (this.property instanceof ObjectProperty) {
            // This is likely a placeholder around Swagger funkiness
            return "Object";
        }
        return "UNKNOWN";
    }

    public String getJavascriptTypeRef() {
        if (this.property instanceof StringProperty) {
            return "String";
        } else if (this.property instanceof DateTimeProperty) {
            return "Date";
        } else if (this.property instanceof ArrayProperty) {
            ArrayProperty arrayProperty = (ArrayProperty)this.property;
            String type = arrayProperty.getItems().getType();
            String jType = "UNKNOWN";
            if (type.equals("string")) {
                jType = "String";
            } else if (type.equals("integer")) {
                jType = "Number";
            } else if (arrayProperty.getItems() instanceof RefProperty){
                RefProperty refProperty = (RefProperty)arrayProperty.getItems();
                jType = javascriptModelRef(refProperty.getSimpleRef());
            } else if (arrayProperty.getItems() instanceof MapProperty) {
                MapProperty mapProperty = (MapProperty)arrayProperty.getItems();
                jType = mapProperty.getType();
            }
            return "Array of " + jType;
        } else if (this.property instanceof BaseIntegerProperty) {
            return "Number";
        } else if (this.property instanceof RefProperty) {
            RefProperty refProperty = (RefProperty)this.property;
            return javascriptModelRef(refProperty.getSimpleRef());
        } else if (this.property instanceof ObjectProperty) {
            // This is likely a placeholder around Swagger funkiness
            return "Object";
        }
        return "UNKNOWN";
    }



    public String getRubyTypeRef() {
        if (this.property instanceof StringProperty) {
            return "String";
        } else if (this.property instanceof DateTimeProperty) {
            return "Date";
        } else if (this.property instanceof ArrayProperty) {
            ArrayProperty arrayProperty = (ArrayProperty)this.property;
            String type = arrayProperty.getItems().getType();
            String jType = "UNKNOWN";
            if (type.equals("string")) {
                jType = "String";
            } else if (type.equals("integer")) {
                jType = "Number";
            } else if (arrayProperty.getItems() instanceof RefProperty){
                RefProperty refProperty = (RefProperty)arrayProperty.getItems();
                jType = rubyModelRef(refProperty.getSimpleRef());
            } else if (arrayProperty.getItems() instanceof MapProperty) {
                MapProperty mapProperty = (MapProperty)arrayProperty.getItems();
                jType = mapProperty.getType();
            }
            return "Array of " + jType;
        } else if (this.property instanceof BaseIntegerProperty) {
            return "Number";
        } else if (this.property instanceof RefProperty) {
            RefProperty refProperty = (RefProperty)this.property;
            return rubyModelRef(refProperty.getSimpleRef());
        } else if (this.property instanceof ObjectProperty) {
            // This is likely a placeholder around Swagger funkiness
            return "Object";
        }
        return "UNKNOWN";
    }

    public String getPhpTypeRef() {
        if (this.property instanceof StringProperty) {
            return "string";
        } else if (this.property instanceof DateTimeProperty) {
            return "\\DateTime";
        } else if (this.property instanceof ArrayProperty) {
            ArrayProperty arrayProperty = (ArrayProperty)this.property;
            String type = arrayProperty.getItems().getType();
            String jType = "UNKNOWN";
            if (type.equals("string")) {
                jType = "string";
            } else if (type.equals("integer")) {
                jType = "integer";
            } else if (arrayProperty.getItems() instanceof RefProperty){
                RefProperty refProperty = (RefProperty)arrayProperty.getItems();
                jType = phpModelRef(refProperty.getSimpleRef());
            } else if (arrayProperty.getItems() instanceof MapProperty) {
                MapProperty mapProperty = (MapProperty)arrayProperty.getItems();
                jType = mapProperty.getType();
            }
            return "Array of " + jType;
        } else if (this.property instanceof BaseIntegerProperty) {
            return "integer";
        } else if (this.property instanceof RefProperty) {
            RefProperty refProperty = (RefProperty)this.property;
            return phpModelRef(refProperty.getSimpleRef());
        } else if (this.property instanceof ObjectProperty) {
            // This is likely a placeholder around Swagger funkiness
            return "object";
        }
        return "UNKNOWN";
    }

    public String getPythonTypeRef() {
        if (this.property instanceof StringProperty) {
            return "str";
        } else if (this.property instanceof DateTimeProperty) {
            return "datetime";
        } else if (this.property instanceof ArrayProperty) {
            ArrayProperty arrayProperty = (ArrayProperty)this.property;
            String type = arrayProperty.getItems().getType();
            String jType = "UNKNOWN";
            if (type.equals("string")) {
                jType = "str";
            } else if (type.equals("integer")) {
                jType = "int";
            } else if (arrayProperty.getItems() instanceof RefProperty){
                RefProperty refProperty = (RefProperty)arrayProperty.getItems();
                jType = pythonModelRef(refProperty.getSimpleRef());
            } else if (arrayProperty.getItems() instanceof MapProperty) {
                MapProperty mapProperty = (MapProperty)arrayProperty.getItems();
                jType = mapProperty.getType();
            }
            return "list[" + jType + "]";
        } else if (this.property instanceof BaseIntegerProperty) {
            return "int";
        } else if (this.property instanceof RefProperty) {
            RefProperty refProperty = (RefProperty)this.property;
            return pythonModelRef(refProperty.getSimpleRef());
        } else if (this.property instanceof ObjectProperty) {
            // This is likely a placeholder around Swagger funkiness
            return "object";
        }
        return "UNKNOWN";
    }

    public boolean isJsDateType() {
        return this.property instanceof DateTimeProperty;
    }
}
